==> bagbag-0.60.49.tar.gz: bagbag-0.60.49/src/bagbag/Tools/Twitter/Nitter_src.py <==
# from ... import Http, File

from bagbag import * 

import typing 

# resp = Http.Get("https://nitter.kavin.rocks/search?f=tweets&q=psychologist+a+paradox+&since=&until=&near=")

# File("tweet.html").Write(resp.Content)

# data = File("tweet.html").Read()

# x = Tools.XPath(data)

# item = x.Find("/html/body/div/div/div[2]/div[1]")

class NitterTwitterUser():
    def __init__(self) -> None:
        self.Name:str = None 
        self.ScreenName:str = None 
    
    def __repr__(self) -> str:
        return f"NitterTwitterUser(Name={self.Name} ScreenName={self.ScreenName})"

    def __str__(self) -> str:
        return self.__repr__()

class NitterTweet():
    def __init__(self) -> None:
        self.ID:int = None
        self.User:NitterTwitterUser = None 
        self.Time:int = None 
        self.Text:str = None 
        self.FavoriteCount:int = None 
        self.RetweetCount:int = None 
        self.CommentCount:int = None 
        self.WebURL:str = None 
        
        # self.in_reply_to_tweet_id:int = None
        # self.twitter:Nitter = None 

    def __repr__(self) -> str:
        return f"NitterTweet(ID={self.ID} Time={self.Time} Text={self.Text} User={self.User} FavoriteCount={self.FavoriteCount} RetweetCount={self.RetweetCount} CommentCount={self.CommentCount} WebURL={self.WebURL})"
    
    def __str__(self) -> str:
        return self.__repr__()

class Nitter():
    def __init__(self) -> None:
        pass 

    def parseTweetHtml(self, html:str) -> list[NitterTweet]:
        # j = Json.Loads(html)

        # for item in j['html'][0]['body'][0]['div'][0]['div'][0]['div'][1]['div']:
        #     username = item['div'][0]['div'][0]['div'][0]['div'][0]['div'][0]['a'][0]["_value"]
        #     screenname = item['div'][0]['div'][0]['div'][0]['div'][0]['div'][0]['a'][1]["_value"]

        nts = []

        x = Tools.XPath(html)

        for idx in Range(1, 99):
            # /html/body/div/div/div[2]/div[1]
            # /html/body/div/div/div[2]/div[2]
            # /html/body/div/div/div[2]/div[1]/div/div[1]/div/div/div/a[1]
            basex = f"/html/body/div/div/div[2]/div[{idx}]"

            if x.Find(f"{basex}/div/div[1]/div/div/div/a[1]") == None:
                break 

            Lg.Trace("/html/body/div/div/div[2]/div[1]/div/div[1]/div/div/div/a[1]")
            Lg.Trace(f"{basex}/div/div[1]/div/div/div/a[1]")

            tid = x.Find(f"{basex}/a").Attribute("href").split("#")[0].split("/")[-1]

            username = x.Find(f"{basex}/div/div[1]/div/div/div/a[1]").Text()
            screenname = x.Find(f"{basex}/div/div[1]/div/div/div/a[2]").Text()[1:]

            text = ""
            section1 = x.Find(f"{basex}/div/div[2]")
            if section1.Attribute("class") == "replying-to":
                text = section1.Text().replace("Replying to", "RE") + " "
                textsidx = 3
            else:
                textsidx = 2

            text = text + x.Find(f"{basex}/div/div[{textsidx}]").Text()
            time = Time.Strptime(x.Find(f"{basex}/div/div[1]/div/div/span/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
            # /html/body/div/div/div[2]/div[5]/div/div[4]/span[1]
            # /html/body/div/div/div[2]/div[2]/div/div[4]
            comment = x.Find(f"{basex}/div/div[4]/span[1]").Text().strip()
            comment = int(comment) if String(comment).IsDigit() else 0

            retweet = x.Find(f"{basex}/div/div[4]/span[2]").Text().strip()
            retweet = int(retweet) if String(retweet).IsDigit() else 0

            quota = x.Find(f"{basex}/div/div[4]/span[3]").Text().strip()
            quota = int(quota) if String(quota).IsDigit() else 0

            like = x.Find(f"{basex}/div/div[4]/span[4]").Text().strip()
            like = int(like) if String(like).IsDigit() else 0

            ntu = NitterTwitterUser()
            ntu.Name = username 
            ntu.ScreenName = screenname 

            nt = NitterTweet()
            nt.CommentCount = comment
            nt.FavoriteCount = like
            nt.ID = tid 
            nt.RetweetCount = retweet
            nt.Text = text 
            nt.Time = time 
            nt.User = ntu 
            nt.WebURL = f"https://twitter.com/{nt.User.ScreenName}/status/{nt.ID}"

            nts.append(nt)

            # Lg.Trace({
            #     "tid": tid,
            #     "username": username,
            #     "screenname": screenname,
            #     "text": text,
            #     "time": time,
            #     "comment": comment, 
            #     "retweet": retweet,
            #     "quota": quota,
            #     "like": like
            # })

        return nts
    
    def getNextPageLink(self, html:str) -> str | None:
        url = String(html).RegexFind('<div class="show-more"><a href="(.+?)">Load more</a></div>')
        if len(url) == 0:
            return None 
        
        

        return url[0][1]

    
    def Search(self, key:str) -> typing.Iterator[NitterTweet]:
        pass 
            
if __name__ == "__main__":
    n = Nitter()
    
    html = File("tweet.html").Read()

    # for i in n.parseTweetHtml(html):
    #     Lg.Trace(i)

    Lg.Trace(n.getNextPageLink(html))

==> bagbag-0.60.50.tar.gz: bagbag-0.60.50/src/bagbag/Tools/Twitter/Nitter_src.py <==
# from ... import Http, File

from bagbag import * 

import typing 

# resp = Http.Get("https://nitter.kavin.rocks/search?f=tweets&q=psychologist+a+paradox+&since=&until=&near=")

# File("tweet.html").Write(resp.Content)

# data = File("tweet.html").Read()

# x = Tools.XPath(data)

# item = x.Find("/html/body/div/div/div[2]/div[1]")

class NitterTwitterUser():
    def __init__(self) -> None:
        self.Name:str = None 
        self.ScreenName:str = None 

    def setStat(self, html:str):
        x = Tools.XPath(html)
                                # /html/body/div/div/div[2]/div/div[2]/div[2]/ul/li[1]/span[2]
        tweetsCount = int(x.Find("/html/body/div/div/div[2]/div[1]/div[2]/div[5]/ul/li[1]/span[2]").Text().replace(",", ""))
        followingCount = int(x.Find("/html/body/div/div/div[2]/div[1]/div[2]/div[5]/ul/li[2]/span[2]").Text().replace(",", ""))
        followersCount = int(x.Find("/html/body/div/div/div[2]/div[1]/div[2]/div[5]/ul/li[3]/span[2]").Text().replace(",", ""))
        likesCount = int(x.Find("/html/body/div/div/div[2]/div[1]/div[2]/div[5]/ul/li[4]/span[2]").Text().replace(",", ""))

        Lg.Trace({
            "tweetsCount": tweetsCount,
            "followingCount": followingCount,
            "followersCount": followersCount,
            "likesCount": likesCount,
        })
    
    def __repr__(self) -> str:
        return f"NitterTwitterUser(Name={self.Name} ScreenName={self.ScreenName})"

    def __str__(self) -> str:
        return self.__repr__()

class NitterTweet():
    def __init__(self) -> None:
        self.ID:int = None
        self.User:NitterTwitterUser = None 
        self.Time:int = None 
        self.Text:str = None 
        self.FavoriteCount:int = None 
        self.RetweetCount:int = None 
        self.CommentCount:int = None 
        self.WebURL:str = None 
        
        # self.in_reply_to_tweet_id:int = None
        # self.twitter:Nitter = None 

    def __repr__(self) -> str:
        return f"NitterTweet(ID={self.ID} Time={self.Time} Text={self.Text} User={self.User} FavoriteCount={self.FavoriteCount} RetweetCount={self.RetweetCount} CommentCount={self.CommentCount} WebURL={self.WebURL})"
    
    def __str__(self) -> str:
        return self.__repr__()

class Nitter():
    def __init__(self, server:str) -> None:
        self.server = server.rstrip("/") 

        if not self.server.startswith("http://") and not self.server.startswith("https://"):
            self.server = 'http://' + self.server

    def getTweet(self, html:str) -> list[NitterTweet]:
        # j = Json.Loads(html)

        # for item in j['html'][0]['body'][0]['div'][0]['div'][0]['div'][1]['div']:
        #     username = item['div'][0]['div'][0]['div'][0]['div'][0]['div'][0]['a'][0]["_value"]
        #     screenname = item['div'][0]['div'][0]['div'][0]['div'][0]['div'][0]['a'][1]["_value"]

        nts = []

        x = Tools.XPath(html)

        for idx in Range(1, 99):
            # /html/body/div/div/div[2]/div[1]
            # /html/body/div/div/div[2]/div[2]
            # /html/body/div/div/div[2]/div[1]/div/div[1]/div/div/div/a[1]
            basex = f"/html/body/div/div/div[2]/div[{idx}]"

            if x.Find(f"{basex}/div/div[1]/div/div/div/a[1]") == None:
                break 

            # Lg.Trace("/html/body/div/div/div[2]/div[1]/div/div[1]/div/div/div/a[1]")
            # Lg.Trace(f"{basex}/div/div[1]/div/div/div/a[1]")

            tid = x.Find(f"{basex}/a").Attribute("href").split("#")[0].split("/")[-1]

            username = x.Find(f"{basex}/div/div[1]/div/div/div/a[1]").Text()
            screenname = x.Find(f"{basex}/div/div[1]/div/div/div/a[2]").Text()[1:]

            text = ""
            section1 = x.Find(f"{basex}/div/div[2]")
            if section1.Attribute("class") == "replying-to":
                text = section1.Text().replace("Replying to", "RE") + " "
                textsidx = 3
            else:
                textsidx = 2

            text = text + x.Find(f"{basex}/div/div[{textsidx}]").Text()
            time = Time.Strptime(x.Find(f"{basex}/div/div[1]/div/div/span/a").Attribute("title"), "%b %d, %Y · %I:%M %p UTC")
            
            # /html/body/div/div/div[2]/div[5]/div/div[4]/span[1]
            # /html/body/div/div/div[2]/div[2]/div/div[4]
            comment = x.Find(f"{basex}/div/div[4]/span[1]").Text().strip()
            comment = int(comment) if String(comment).IsDigit() else 0

            retweet = x.Find(f"{basex}/div/div[4]/span[2]").Text().strip()
            retweet = int(retweet) if String(retweet).IsDigit() else 0

            quota = x.Find(f"{basex}/div/div[4]/span[3]").Text().strip()
            quota = int(quota) if String(quota).IsDigit() else 0

            like = x.Find(f"{basex}/div/div[4]/span[4]").Text().strip()
            like = int(like) if String(like).IsDigit() else 0

            ntu = NitterTwitterUser()
            ntu.Name = username 
            ntu.ScreenName = screenname 

            nt = NitterTweet()
            nt.CommentCount = comment
            nt.FavoriteCount = like
            nt.ID = tid 
            nt.RetweetCount = retweet
            nt.Text = text 
            nt.Time = time 
            nt.User = ntu 
            nt.WebURL = f"https://twitter.com/{nt.User.ScreenName}/status/{nt.ID}"

            nts.append(nt)

            # Lg.Trace({
            #     "tid": tid,
            #     "username": username,
            #     "screenname": screenname,
            #     "text": text,
            #     "time": time,
            #     "comment": comment, 
            #     "retweet": retweet,
            #     "quota": quota,
            #     "like": like
            # })

        return nts
    
    def getNextPageLink(self, html:str) -> str | None:
        url = String(html).RegexFind('<div class="show-more"><a href="(.+?)">Load more</a></div>')
        if len(url) == 0:
            return None   

        return self.server + "/search" + String(url[0][1]).HTMLDecode()

    def getHTML(self, url:str) -> str:
        resp = Http.Get(url)
        return resp.Content

    def Search(self, key:str) -> typing.Iterator[NitterTweet]:
        url = self.server + "/search?f=tweet&q=" + String(key).URLEncode()

        while True:
            html = self.getHTML(url)

            for t in self.getTweet(html):
                yield t 

            url = self.getNextPageLink(html)

            if url == None:
                break 
            
if __name__ == "__main__":
    # ---- 调试 NitterTwitterUser

    # 活动频繁
    # html = Http.Get("https://nitter.kavin.rocks/blahlaja").Content
    # File("user.html").Write(html)

    # 没有推文
    # html = Http.Get("https://nitter.kavin.rocks/binsingha").Content
    # File("user.1.html").Write(html)

    html = File("user.1.html").Read()

    ntu = NitterTwitterUser()
    ntu.setStat(html)

    # ---- 联调 Nitter 

    # n = Nitter("nitter.kavin.rocks")

    # for t in n.Search("psychologist a paradox"):
    #     Lg.Trace(t)

    # ---- 调试 Nitter 
    
    # html = File("tweet.html").Read()

    # for i in n.parseTweetHtml(html):
    #     Lg.Trace(i)

    # html = Http.Get("https://nitter.kavin.rocks/search?f=tweets&q=psychologist+a+paradox+&cursor=1677510929099923455%26").Content

    # Lg.Trace(n.getNextPageLink(html))
